"use strict";

/* ====================================================================================
			SCRIPT NAMES
   ==================================================================================== */

this.name            = "AI Trading Assistant";
this.version         = "2.12";
this.$buyingMessage  = "AI Buying Advisor";
this.$sellingMessage = "AI Selling Advisor";
this.$tradingEngine  = "AI Autotrade";

/* ====================================================================================
			STARTING MESSAGES

	These are the only text strings defined in this script. See 
	missiontext.plist file for the opportunity cost (number of dockings) warning.
   ==================================================================================== */

this.$startMessage   = ["Trading Assistant", "Hello.", ""];
this.$fuelMessage    = ["Trading Assistant", "We need fuel.", "fuel"];

/* ====================================================================================
			LOOK-UP TABLE FOR _setDisplayMode FUNCTION
   ==================================================================================== */

this.$displayMode = [[1],
    ["average cost",     this.name, "_averagesPaidDisplay",    this.name, "_setCostHUD"],
    ["opportunity cost", this.name, "_opportunityCostDisplay", this.name, "_setDocksHUD"],
    ["arbitrage",        this.name, "_arbitrageDisplay",       this.name, "_setDiffHUD"],
    ["buy or sell",      this.name, "_buyOrSellDisplay",       this.name, "_setAutoHUD"],
    ["blank",            this.name, "_blankDisplay",           this.name, "_setBlankHUD"]];

/* ====================================================================================
			CONVERSION TABLE FOR COMMODITY NAMES TO LEDGER NUMBERS
   ==================================================================================== */

this.$commodities = ["food", "textiles", "radioactives", "slaves", "liquor_wines", "luxuries", "narcotics", "computers", "machinery", "alloys", "firearms", "furs", "minerals", "gold", "platinum", "gem_stones", "alien_items"];

/* ====================================================================================
			RECORD OF OPEN TRADES & LONG-RUN ACCOUNTS
   ==================================================================================== */

this.$dockingHistory = [[]]; // Two dimensional array. Should always exist.

    this.startUp = function () {
        "use strict";
        var arrayLength = this.$commodities.length;
        var i, cargo, quantity, price;

        if (missionVariables.tradingAI_longRun) {
            this.$longRunAccounts = JSON.parse(missionVariables.tradingAI_longRun);
        } else {
            this.$longRunAccounts = [[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0],[0,0,0]];
        }

        if (missionVariables.tradingAI_totalsBought &&
            missionVariables.tradingAI_totalsSold) {
            this.$totalsBought = JSON.parse(missionVariables.tradingAI_totalsBought);
            this.$totalsSold   = JSON.parse(missionVariables.tradingAI_totalsSold);
        } else { // Copy $totalsBought from manifest...
            this.$totalsBought = new Array(arrayLength);
            for (i = 0; i < arrayLength; i++) {
                cargo = this.$commodities[i];
                this.$totalsBought[i] = manifest[cargo];
            }
            this.$totalsSold = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];
        }

        if (missionVariables.tradingAI_ledger) {
            this.$ledger = JSON.parse(missionVariables.tradingAI_ledger);
        } else { // First run, use market prices to set ledgers...
            this.$ledger = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];
            for (i = 0; i < arrayLength; i++) {
                cargo = this.$commodities[i];
                quantity = manifest[cargo];
                if (quantity > 0) {
                    price = system.mainStation.market[cargo].price;
                    this.$ledger[i] = quantity * price;
                }
            }
        }

        if (missionVariables.tradingAI_oppCosts) {
            this.$opportunityCosts = JSON.parse(missionVariables.tradingAI_oppCosts);
        } else {
            this.$opportunityCosts = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];
        }
        if (missionVariables.tradingAI_dockingHistory) {
            this.$dockingHistory = JSON.parse(missionVariables.tradingAI_dockingHistory);
        }
        this.$lastStationVisited = system.mainStation;

        this.$averagesPaid = new Array(17);
        this._calculateAveragesPaid();
        this.$tradingMessages = false;
        delete this.startUp;
    }

    this.playerWillSaveGame = function () {
        "use strict";
        missionVariables.tradingAI_ledger         = JSON.stringify(this.$ledger);
        missionVariables.tradingAI_longRun        = JSON.stringify(this.$longRunAccounts);
        missionVariables.tradingAI_totalsBought   = JSON.stringify(this.$totalsBought);
        missionVariables.tradingAI_totalsSold     = JSON.stringify(this.$totalsSold);
        missionVariables.tradingAI_oppCosts       = JSON.stringify(this.$opportunityCosts);
        missionVariables.tradingAI_dockingHistory = JSON.stringify(this.$dockingHistory);
    }

/* ====================================================================================
			ACTIVATED ON CHANGE TO MARKET SCREEN
   ==================================================================================== */

    this.guiScreenChanged = function (to, from) {
        "use strict";
        if (to === "GUI_SCREEN_MARKET" && // <<----------------------------------------------------------------- NEW!
            player.ship.docked &&
            guiScreen === "GUI_SCREEN_MARKET" &&
            player.ship.equipmentStatus("EQ_AI_TRADING_ASSISTANT") === "EQUIPMENT_OK") {
            this._startTradingDisplay();
            worldScripts[this.$tradingEngine]._startAutotrade();
            return;
        }
        if (from === "GUI_SCREEN_MARKET" && this.$originalHUD) {
            this._stopTradingDisplay();
            worldScripts[this.$tradingEngine]._stopAutotrade();
        }
    }

    this._startTradingDisplay = function () {
        "use strict";
        this.$tradingMessages = true;
        this.$originalHUD = player.ship.hud;
        if (player.ship.dockedStation.isMainStation || player.ship.targetSystem !== system.ID) {
            this._setDisplayMode();  // default
        } else {
            this._setDisplayMode(3); // arbitrage
        }
        if (player.ship.fuel == 7) {
            this.$showMessage(this.$startMessage[0], this.$startMessage[1], this.$startMessage[2]);
        } else {
            this.$showMessage(this.$fuelMessage[0], this.$fuelMessage[1], this.$fuelMessage[2]);
        }
        this.$displayRefreshTimer = new Timer(this, this._refreshDisplay, 9, 9);
        this.$sequenceTimer       = new Timer(this, this._thingsToDo, 4, 7);
        this.commsMessageReceived = this.commsMessageReceived_OFF;
    }

    this._stopTradingDisplay = function () {
        "use strict";
        this.$tradingMessages = false;
        if (this.$displayRefreshTimer) {
            this.$displayRefreshTimer.stop();
            delete this.$displayRefreshTimer;
        }
        if (this.$sequenceTimer) {
            this.$sequenceTimer.stop();
            delete this.$sequenceTimer;
        }
        player.ship.hud = this.$originalHUD;
        delete this.$originalHUD;
        delete this.commsMessageReceived;
        this.$displayMode[0] = [this.$displayMode[0].pop()]; // Flush change history.
    }

/* ====================================================================================
			OOLITE 1.77 SCREEN CHANGE BUG WORK-AROUND
   ==================================================================================== */

    this.shipWillLaunchFromStation = function () {
        "use strict";
        if (this.$originalHUD) {
            this._stopTradingDisplay();
        }
    }

/* ====================================================================================
			TO-DO:

	this._startTradingDisplay needs to be de-crufted, an' some of it moved to
	this._thingsToDo, which should be developed into a proper sequencer, 
	rather than a one-time check. this.$sequenceTimer should be left running to
	periodically check the state of the trading engine, and if it's idle, display
	a random tip message, rather than the current bodginess of daisy-chaining
	the tips onto _performTrading in AI Autotrade. 
   ==================================================================================== */

/* ====================================================================================
			SEQUENCE OF THINGS TO DO - OPPORTUNITY COSTS CHECK
   ==================================================================================== */

    this._thingsToDo = function () {
        "use strict";
        if (this.$thingsDone) {
            if (this.$sequenceTimer) {
                this.$sequenceTimer.stop();
                delete this.$sequenceTimer;
            }
            delete this.$thingsDone;
            this._undoDisplayMode(2);
        } else {
            var nuisanceCargo = this._findHighOpportunityCosts();
            if (nuisanceCargo[0]) {
                var opportunities = nuisanceCargo[0][1];
            }
            if (opportunities > 1) {
                var cargo = nuisanceCargo[0][0];
                var message = this._reportOpportunityCost(cargo, opportunities);
                this._setDisplayMode(2, false);
                this.$showMessage(message, "", cargo);
            }
            this.$thingsDone = true;
        }
    }

/* ====================================================================================
			PLAYER BUYING CARGO
   ==================================================================================== */

    this.playerBoughtCargo = function (cargo, quantity, unitPrice) {
        "use strict";
        var ledgerNumber = this.$commodities.indexOf(cargo);
        this.$ledger[ledgerNumber] += (unitPrice * quantity); // Record payments.
        this.$totalsBought[ledgerNumber] += quantity;

        this._updateAveragePricePaid(cargo, ledgerNumber);

        if (this.$tradingMessages) {
            worldScripts[this.$buyingMessage].$reactToPlayerBuyingCargo(cargo, quantity, unitPrice);
        }
    }

/* ====================================================================================
			PLAYER SELLING CARGO
   ==================================================================================== */

    this.playerSoldCargo = function (cargo, quantity, unitPrice) {
        "use strict";
        var ledgerNumber = this.$commodities.indexOf(cargo);
        this.$ledger[ledgerNumber] -= (unitPrice * quantity); // Record receipts.
        this.$totalsSold[ledgerNumber] += quantity;

        var closed = false;
        if (manifest[cargo] == 0) { // Closing book...
            // Transfer balance to long-run account & reset ledger...
            this.$longRunAccounts[ledgerNumber][0] -= this.$ledger[ledgerNumber]; // Yes, that is a minus sign.
            this.$longRunAccounts[ledgerNumber][1] += this.$totalsBought[ledgerNumber];
            this.$longRunAccounts[ledgerNumber][2] += this.$totalsSold[ledgerNumber];
            this.$ledger[ledgerNumber] = 0;
            this.$totalsBought[ledgerNumber] = 0;
            this.$totalsSold[ledgerNumber] = 0;
            this.$opportunityCosts[ledgerNumber] = 0;
            closed = true;
        }

        // Retrieve paid price BEFORE updating average price paid...
        var paidPrice = this.$averagesPaid[ledgerNumber] * 10;
        this._updateAveragePricePaid(cargo, ledgerNumber);

        if (this.$tradingMessages) {
            worldScripts[this.$sellingMessage].$reactToPlayerSellingCargo(cargo, quantity, unitPrice, paidPrice, closed);
        }
    }

/* ====================================================================================
			CALCULATE AVERAGE PRICES PAID
   ==================================================================================== */

    this._calculateAveragesPaid = function () {
        "use strict";
        var arrayLength = this.$commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            this._updateAveragePricePaid(this.$commodities[i], i);
        }
    }

    this._updateAveragePricePaid = function (cargo, ledgerNumber) {
        "use strict";
        var quantity = manifest[cargo];
        if (quantity == 0) {
            this.$averagesPaid[ledgerNumber] = "0.0";
        } else {
            // Average price per unit paid in credits (ledger is in deci-credits)...
            this.$averagesPaid[ledgerNumber] = (this.$ledger[ledgerNumber] / (quantity * 10)).toFixed(1);
        }
    }

/* ====================================================================================
			INCREMENT OPPORTUNITY COSTS AND UPDATE 
			DOCKING HISTORY WITH EACH NEW STATION VISITED
   ==================================================================================== */

    this.shipLaunchedFromStation = function (station) {
        "use strict";
        this.$lastStationVisited = station;
        this.$dockingHistory[0][4] = player.credits;
    }

    this.shipDockedWithStation = function (station) {
        "use strict";
        if (station !== this.$lastStationVisited) {
            var highest = this._incrementOpportunityCosts();
            this.$dockingHistory = this.$dockingHistory.slice(0, highest);
            var description = system.name + " \u2014 " + station.displayName;
            if (system.info.sun_name) {
                description += "  (" + system.info.sun_name + " system)";
            }
            this.$dockingHistory.unshift([galaxyNumber, system.ID, description, player.credits, player.credits]);
        }
    }

    this._incrementOpportunityCosts = function () {
        "use strict";
        var highest = 0;
        var arrayLength = this.$commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            if (manifest[this.$commodities[i]] > 0) {
                this.$opportunityCosts[i] += 1;
                if (this.$opportunityCosts[i] > highest) {
                    highest = this.$opportunityCosts[i];
                }
            } else {
                this.$opportunityCosts[i] = 0;
            }
        }
        return highest;
    }

/* ====================================================================================
			SCAN MANIFEST FOR HIGH OPPORTUNITY COSTS
   ==================================================================================== */

    this._findHighOpportunityCosts = function () {
        "use strict";
    
        function descendingSort(a, b) {
            return b[1] - a[1];
        }
    
        var nuisanceCargo = new Array;
        var cargo;
        var opportunities;
        var arrayLength = this.$commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            cargo = this.$commodities[i];
            opportunities = this.$opportunityCosts[i];
            if (manifest[cargo] > 0 && opportunities > 0) {
                switch (cargo) {
                  case "gold":
                  case "platinum":
                  case "gem_stones":
                    if (manifest[cargo] > 63) {
                        nuisanceCargo.push([cargo, opportunities]);
                    }
                    break;
                  default:
                    nuisanceCargo.push([cargo, opportunities]);
                    break;
                }
            }
        }
        nuisanceCargo.sort(descendingSort);
        return nuisanceCargo;
    }

/* ====================================================================================
			REPORT OPPORTUNITY COST OF SPECIFIC CARGO
   ==================================================================================== */

    this._reportOpportunityCost = function (cargo, opportunities) {
        "use strict";
        if (!cargo) {
            return undefined;
        }
        if (!opportunities) {
            var ledgerNumber = this.$commodities.indexOf(cargo);
            var opportunities = this.$opportunityCosts[ledgerNumber];
            if (opportunities < 1) {
                return false;
            }
        }
        var message;
        var ordinalNumbers = ["no", "first", "second", "third", "fourth", "fifth", "sixth", "seventh", "eighth", "ninth", "tenth", "eleventh", "twelfth"];
        var displayName = displayNameForCommodity(cargo);
        if (ordinalNumbers.length > opportunities) {
            var ordinal = ordinalNumbers[opportunities];
            message = expandMissionText("tradingAI_oppcost_number", {tradingAI_ordinal_number: ordinal, tradingAI_Commodity: displayName, tradingAI_commodity: displayName.toLowerCase()});
        } else {
            message = expandMissionText("tradingAI_oppcost_many", {tradingAI_Commodity: displayName, tradingAI_commodity: displayName.toLowerCase()});
        }
        return message;
    }

/* ====================================================================================
			SHOW MESSAGE - THS IS THE FUNCTION OTHER SCRIPTS SHOULD CALL

worldScripts["AI Trading Assistant"].$showMessage(message, summary, highlight);

message   - is the comms message to appear in the lower left of the screen.
summary   - is the one-word message that may appear at the bottom of the costs column.
highlight - is the commodity type to be highlighted by a HUD change, one of this.$commodities (or "fuel").

   ==================================================================================== */

    this.$showMessage = function (message, bottomLine, highlight) {
        "use strict";
        if (this.$displayRefreshTimer) {
            this.$displayRefreshTimer.stop();
            this.$displayRefreshTimer.nextTime = clock.absoluteSeconds + 8;
            this.$displayRefreshTimer.start();
        }
        this._setHUD(highlight);
        player.commsMessage("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n" + message, 5);
        if (!bottomLine) {
            bottomLine = "    ";
        }
        player.consoleMessage(this._columnDisplay(), 10);
        player.consoleMessage("\n" + bottomLine, 3);
    }

    this._refreshDisplay = function () {
        player.consoleMessage(this._columnDisplay(), 10);
        player.consoleMessage("\n    ", 3);
    }

    this.commsMessageReceived_OFF = function () {
        "use strict";
        worldScripts[this.$tradingEngine]._pauseAutotrade(15);
        if (this.$sequenceTimer && this.$sequenceTimer.isRunning) {
            this.$sequenceTimer.stop();
            this.$sequenceTimer.nextTime = clock.absoluteSeconds + 15;
            this.$sequenceTimer.start();
        }
        if (this.$displayRefreshTimer) {
            this.$displayRefreshTimer.stop();
            this.$displayRefreshTimer.nextTime = clock.absoluteSeconds + 8;
            this.$displayRefreshTimer.start();
        }
        this._refreshDisplay();
    }

/* ====================================================================================
			SET DISPLAY MODE TO USE
   ==================================================================================== */

    this._resetDisplayMode = function (mode) {
        "use strict";
        if (!mode) {
            mode = this.$displayMode[0].pop();
        }
        if (mode > 0 && mode < this.$displayMode.length) {
            this.$displayMode[0] = [mode];
            return true;
        } else {
            this.$displayMode[0] = [1];
            return false;
        }
    }

    this._setDisplayMode = function (mode, immediately) {
        "use strict";
        var specified = true;
        if (!mode) {
            mode = this.$displayMode[0][0];
            specified = false;
        }
        if (this.$displayMode[mode]) {
            var modeName = this.$displayMode[mode][0];
            var columnScript = this.$displayMode[mode][1];
            var columnFunction = this.$displayMode[mode][2];
            this._columnDisplay = worldScripts[columnScript][columnFunction];
            if (this.$displayMode[mode][4]) {
                var hudScript = this.$displayMode[mode][3];
                var hudFunction = this.$displayMode[mode][4];
                this._setHUD = worldScripts[hudScript][hudFunction];
            }
            if (specified) {
                this.$displayMode[0].unshift(mode);
            }
            if (immediately) {
                this._setHUD("");
                this._refreshDisplay();
            }
            return modeName;
        } else {
            return false;
        }
    }

    this._undoDisplayMode = function (mode) {
        "use strict";
        if (mode && this.$displayMode[0].length > 1) {
            var index = this.$displayMode[0].indexOf(mode);
            if (index === -1) {
                return null;
            } else {
                this.$displayMode[0].splice(index, 1);
                this._setDisplayMode();
                return true;
            }
        } else {
            return false;
        }
    }

    this._addDisplayMode = function (newMode) {
        "use strict";
        if (newMode && newMode[4]) {
        //  var modeName = newMode[0];
            var columnScript = newMode[1];
            var columnFunction = newMode[2];
            if (!worldScripts[columnScript][columnFunction]) {
                return false;
            }
            var hudScript = newMode[3];
            var hudFunction = newMode[4];
            if (!worldScripts[hudScript][hudFunction]) {
                return false;
            }
            this.$displayMode.push(newMode);
            return this.$displayMode.length -1;
        } else {
            return false;
        }
    }

/* ====================================================================================
			DISPLAY COLUMN OF NUMBERS AS CONSOLE MESSAGE
   ==================================================================================== */

    this._averagesPaidDisplay = function () {
        "use strict";
        var message = " ";
        var padding;
        var arrayLength = this.$averagesPaid.length;
        for (var i = 0; i < arrayLength; i++) {
            if (this.$averagesPaid[i] < 0.1) {
                message = message + "\n    \u2014";
            } else {
                padding = "";
                if (this.$averagesPaid[i] < 10) {
                    padding = "    ";
                } else if (this.$averagesPaid[i] < 100) {
                    padding = "  ";
                }
                message = message + "\n" + padding + this.$averagesPaid[i];
            }
        }
        return message;
    }

    this._opportunityCostDisplay = function () {
        "use strict";
        var message = " ";
        var padding;
        var arrayLength = this.$opportunityCosts.length;
        for (var i = 0; i < arrayLength; i++) {
            if (this.$opportunityCosts[i] < 1) {
                message = message + "\n";
            } else {
                padding = "";
                if (this.$opportunityCosts[i] < 10) {
                    padding = "    ";
                } else if (this.$opportunityCosts[i] < 100) {
                    padding = "  ";
                }
                message = message + "\n" + padding + this.$opportunityCosts[i];
            }
        }
        return message;
    }

    this._arbitrageDisplay = function () {
        "use strict";
        if (player.ship.dockedStation.isMainStation) {
            return "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n";
        } else {
            var message = " ";
            var padding;
            var thatMarket = system.mainStation.market;
            var thisMarket = player.ship.dockedStation.market;
            var arrayLength = this.$commodities.length;
            for (var i = 0; i < arrayLength; i++) {
                var commodity = this.$commodities[i];
                var thatPrice = thatMarket[commodity].price;
                var thisPrice = thisMarket[commodity].price;
                if (thisMarket[commodity].quantity === 0 &&
                    manifest[commodity] === 0) {
                    message = message + "\n";
                } else if (thisPrice === thatPrice) {
                    message = message + "\n    0   ";
                } else{
                    padding = "";
                    var diff = ((thatPrice - thisPrice)/10).toFixed(1);
                    if (diff < -10) {
                        padding = "  ";
                    } else if (diff < 0) {
                        padding = "    ";
                    } else if (diff < 10) {
                        padding = "   +";
                    } else if (diff < 100) {
                        padding = " +";
                    }
                    message = message + "\n" + padding + diff;
                }
            }
        }
        return message;
    }

    this._buyOrSellDisplay = function () {
        "use strict";
        var market = player.ship.dockedStation.market;
        var message = " ";
        var commodity, price, expected, paid, available, padding;
        var arrayLength = this.$commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            commodity = this.$commodities[i];
            price = market[commodity].price;
            expected = worldScripts[this.$tradingEngine].$expectedPrices[i];
            paid = this.$averagesPaid[i];
            available = market[commodity].quantity;
            if (available > 0 && price < expected) {
                message += "\n  Buy";
            } else if (manifest[commodity] > 0 && available < 127 && price > expected && price / 10 > paid) {
                message += "\n  Sell";
            } else if (paid < 0.1) {
                message += "\n    \u2014";
            } else {
                padding = "";
                if (paid < 10) {
                    padding = "    ";
                } else if (paid < 100) {
                    padding = "  ";
                }
                message += "\n" + padding + paid;
            }
        }
        return message;
    }

    this._blankDisplay = function () {
        "use strict";
        return "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n";
    }

/* ====================================================================================
			SELECT HUD FILE TO USE
   ==================================================================================== */

    this._setAutoHUD = function (highlight) {
        "use strict";
        if (guiScreen === "GUI_SCREEN_MARKET") {
            if (!player.ship.dockedStation.isMainStation &&
                (highlight === "slaves" ||
                highlight === "narcotics" || highlight === "firearms")) {
                player.ship.hud = "trading-assistant-auto-" + highlight + "-rh-hud.plist";
            } else {
                player.ship.hud = "trading-assistant-auto-" + highlight + "-hud.plist";
            }
        }
    }

    this._setBlankHUD = function (highlight) {
        "use strict";
        if (guiScreen === "GUI_SCREEN_MARKET") {
            player.ship.hud = "trading-assistant-blank-hud.plist";
        }
    }

    this._setCostHUD = function (highlight) {
        "use strict";
    //    if (!highlight) {
    //        highlight = "default";
    //    }
        if (guiScreen === "GUI_SCREEN_MARKET") {
            if (!player.ship.dockedStation.isMainStation &&
                (highlight === "slaves" ||
                highlight === "narcotics" || highlight === "firearms")) {
                player.ship.hud = "trading-assistant-rh-" + highlight + "-hud.plist";
            } else {
                player.ship.hud = "trading-assistant-galcop-" + highlight + "-hud.plist";
            }
        }
    }

    this._setDiffHUD = function (highlight) {
        "use strict";
        if (guiScreen === "GUI_SCREEN_MARKET") {
            if (highlight === "fuel") {
                player.ship.hud = "trading-assistant-diff-fuel-hud.plist";
            } else {
                player.ship.hud = "trading-assistant-diff-hud.plist";
            }
        }
    }

    this._setDocksHUD = function (highlight) {
        "use strict";
    //    if (!highlight) {
    //        highlight = "default";
    //    }
        if (guiScreen === "GUI_SCREEN_MARKET") {
            player.ship.hud = "trading-assistant-docks-" + highlight + "-hud.plist";
        }
    }

/* ====================================================================================
			UTILITY FUNCTIONS FOR OTHER SCRIPTS TO USE
   ==================================================================================== */

    this._valueManifest = function (market) {
        "use strict";
        if (!market) {
            if (player.ship.docked) {
                market = player.ship.dockedStation.market;
            } else {
                return false;
            }
        }
        var totalValue = 0;
        var cargo, quantity, value;
        var arrayLength = this.$commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            cargo = this.$commodities[i];
            quantity = manifest[cargo];
            if (quantity > 0) {
                value = market[cargo].price * quantity;
                totalValue += value;
            }
        }
        return totalValue; // deci-credits.
    }

    this._totalLedgerCosts = function () {
        "use strict";
        var totalCosts = 0;
        var arrayLength = this.$ledger.length;
        for (var i = 0; i < arrayLength; i++) {
            totalCosts += this.$ledger[i];
        }
        return totalCosts; // deci-credits.
    }

/* ====================================================================================
			COPYRIGHT NOTICE

	(C)2013, Wildeblood. All rights reserved. 
	Free to use, but DO NOT PROMULGATE MODIFIED VERSIONS.

This was created by some fellow with the user name "Wildeblood" on the Oolite forums, 
he'll tell you his real name if you bother to ask.
To use means to play with, modify or re-use in whole or in part, and to distribute freely.
To use DOES NOT imply to use for commercial gain, which would be a breach of copyright.
To promulgate means to make official announcements.
   ==================================================================================== */